"""
Script to embed section-level metadata into the HTML pages.
Such metadata can be detected by citation managers such as Zotero to import metadata automagically. 
In addition, the script transforms the existent BibTeX snippet to a "copy" button. 
"""

# === Imports === 

import re 
import glob 
from os.path import join
from os.path import basename
import pandas as pd


# === Parameters === 

template_file = "metadata_template.md"
source_file = "metadata_source.tsv" 


# === Files === 

def read_template(): 
    with open(template_file, "r", encoding="utf8") as infile: 
        template = infile.read()
        #print(template)
    return template


def read_source():
    with open(source_file, "r", encoding="utf8") as infile: 
        source = pd.read_csv(infile, sep=";", index_col="identifier")
        #print(source.head())
    return source


def read_html_file(html_file): 
    with open(html_file, "r", encoding="utf8") as infile: 
        html = infile.read()
    return html


def generate_metadata(identifier, template, source):

    #== Add author
    author = source.loc[identifier,"author"]
    metadata = re.sub("#AUTHOR#", author, template)

    #== Add title
    title = source.loc[identifier,"title"]
    metadata = re.sub("#TITLE#", title, metadata)

    #== Add url
    url = source.loc[identifier,"url"]
    url = "https://methods.clsinfra.io/" + url
    metadata = re.sub("#URL#", url, metadata)

    #== Add keywords
    keywords = source.loc[identifier,"keywords"]
    metadata = re.sub("#KEYWORDS#", keywords, metadata)

    #== Check metadata
    #print(metadata)
    return metadata


def embed_metadata(identifier, html, metadata): 
    #print(len(metadata))
    html = re.sub("<title>", metadata +" \n\n<title>", html)
    return html 


def make_copytext(html): 
    try: 
        bibtex = re.findall("<pre><code>(.*?)</code></pre>", html, re.DOTALL)[0]
        #bibtex = re.sub("<pre><code>(.*?)</code></pre>", "", html, re.DOTALL)
        bibtex = re.sub("\n", "", bibtex, re.DOTALL)
        bibtex = re.sub("[ ]+", " ", bibtex, re.DOTALL)
        button = '<button onclick="copyText()">Copy BibTeX</button>\n    <script> \n        function copyText() { \n             navigator.clipboard.writeText \n (\"#BIBTEX#\");         }\n    </script>\n'
        html = re.sub("<pre>", button + "<br/><br/><pre>", html, re.DOTALL)
        html = re.sub("#BIBTEX#", bibtex, html)
        #html = re.sub("<pre>", "", html, re.DOTALL) # This line won't work.
    except: 
        print("make_copytext failed.")
    return html


def save_html(identifier, result):
    filename = join("_book", identifier + ".html")
    with open(filename, "w", encoding="utf8") as outfile: 
        outfile.write(result)


# === Main === 

def main():
    template = read_template()
    source = read_source()
    for html_file in glob.glob(join("_book", "*.html")):
        identifier = basename(html_file).split(".")[0]
        #if identifier in ["general-intro", "postscriptum", "analysis-intro"]: # for testing only
        if identifier in list(source.index):
            print("Working on:", identifier)
            html = read_html_file(html_file)
            if "copyText" not in html: 
                html = make_copytext(html)
                print("-- make_copytext")
            else: 
                print("-- skipping make_copytext")
            if "citation_publisher" not in html: 
                metadata = generate_metadata(identifier, template, source)
                html = embed_metadata(identifier, html, metadata)
                print("-- embed metadata")
            else: 
                print("-- skipping embed_metadata")
            save_html(identifier, html)
main()